library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use work.all;

entity reg_tb is
end reg_tb;

architecture behavior of reg_tb is
  constant cycle_time_c : time:=266 ps;
  constant clock_to_output_c : time:=80 ps;
  constant reset_active_high_c : boolean := false;
  constant reset_active_cycles_c : natural := 2;
  constant register_width_c : integer := 32;

  -- clock_gen
  signal clk_s : std_logic;      
  signal reset_s : std_logic;

  signal ld_s, clr_s : std_logic;
  signal din_s, dout_s : std_logic_vector(register_width_c-1 downto 0);
  
begin
  
  clock_gen : entity work.clock_gen(behavioral)
    generic map (
      TCYC => cycle_time_c,
      RES_ACT_HIGH => reset_active_high_c,
      RES_CYCLES => reset_active_cycles_c )
    port map (
      clk   => clk_s,
      reset => reset_s );
    
  reg : entity work.reg(behavioral)
    generic map (
      tco => clock_to_output_c,
      n => register_width_c )
    port map ( clk => clk_s,
               ld => ld_s,
               clr => clr_s,
               din => din_s,
               dout => dout_s );
    
  testbench : process

    variable err_cnt_v : integer := 0;
    
  begin

    clr_s <= '1';
    ld_s <= '0';
    wait for 1000 ps;

    clr_s <= '0';
    ld_s <= '1';
    din_s <= x"0C0FFEE0";
    wait for 250 ps;
    
    -- case 1
    ld_s <= '1';
    wait for clock_to_output_c;
    assert (dout_s=din_s) report "Test1 Failed!" severity error;
    if (dout_s/=din_s) then
      err_cnt_v := err_cnt_v+1;
    end if;
    
    -- case 2				
    ld_s <= '0';
    din_s <= x"00FEE000";  
    wait for clock_to_output_c;
    assert (dout_s/=din_s) report "Test2 Failed!" severity error;
    if (dout_s=din_s) then
      err_cnt_v := err_cnt_v+1;
    end if;		
    
    -- case 3
    ld_s <= '1';
    wait for cycle_time_c;
    assert (dout_s=din_s) report "Test3 Failed!" severity error;
    if (dout_s/=din_s) then
      err_cnt_v := err_cnt_v+1;
    end if;
    
    -- case 4
    clr_s <= '1';
    wait for clock_to_output_c;
    assert (dout_s=x"00000000") report "Test4 Failed!" severity error;
    if (dout_s/=x"00000000") then
      err_cnt_v := err_cnt_v+1;
    end if;
    
    -- summary of all the tests
    if (err_cnt_v=0) then	
      assert false
        report "Testbench of register completely successfully!"
        severity note;
    else
      assert true
        report "Something wrong, check again pls!"
        severity error;
    end if;
    
    wait;
    
  end process testbench;
  
end behavior;
